@@ -34,6 +34,7 @@ import java.util.HashMap;
 import java.util.Locale;
 import java.util.Map;
 import java.util.TimeZone;
+import java.util.TreeMap;
 import java.util.logging.Formatter;
 import java.util.logging.Level;
 import java.util.logging.LogRecord;
@@ -44,12 +45,12 @@ import org.jboss.logmanager.ExtLogRecord;
 /**
  * Formatter utility methods.
  */
+@SuppressWarnings({"WeakerAccess", "unused"})
 public final class Formatters {
 
     public static final String THREAD_ID = "id";
 
     private static final boolean DEFAULT_TRUNCATE_BEGINNING = false;
-    private static final String NEW_LINE = String.format("%n");
     private static final Pattern PRECISION_INT_PATTERN = Pattern.compile("\\d+");
 
 
@@ -120,7 +121,7 @@ public final class Formatters {
      * @return the substring
      */
     private static String applySegments(final String precision, final String subject) {
-        if (precision == null) {
+        if (precision == null || subject == null) {
             return subject;
         }
 
@@ -546,7 +547,13 @@ public final class Formatters {
      * @return the format step
      */
     public static FormatStep hostnameFormatStep(final boolean leftJustify, final int minimumWidth, final boolean truncateBeginning, final int maximumWidth, final boolean qualified) {
-        return hostnameFormatStep(leftJustify, minimumWidth, truncateBeginning, maximumWidth, qualified ? null : "1");
+        return qualified ? hostnameFormatStep(leftJustify, minimumWidth, truncateBeginning, maximumWidth, null) : new SegmentedFormatStep(leftJustify, minimumWidth, truncateBeginning, maximumWidth, null) {
+            public String getSegmentedSubject(final ExtLogRecord record) {
+                final String hostName = record.getHostName();
+                final int idx = hostName.indexOf('.');
+                return idx == -1 ? hostName :hostName.substring(0, idx);
+            }
+        };
     }
 
     /**
@@ -560,9 +567,30 @@ public final class Formatters {
      * @return the format step
      */
     public static FormatStep hostnameFormatStep(final boolean leftJustify, final int minimumWidth, final boolean truncateBeginning, final int maximumWidth, final String precision) {
-        return new SegmentedFormatStep(leftJustify, minimumWidth, truncateBeginning, maximumWidth, precision) {
+        return new SegmentedFormatStep(leftJustify, minimumWidth, truncateBeginning, maximumWidth, null) {
             public String getSegmentedSubject(final ExtLogRecord record) {
-                return record.getHostName();
+                final String hostName = record.getHostName();
+                // Check for a specified precision. This is not passed to the constructor because we want truncate
+                // segments from the right intsead of the left.
+                if (precision != null && PRECISION_INT_PATTERN.matcher(precision).matches()) {
+                    int count = Integer.parseInt(precision);
+                    int end = 0;
+                    for (int i = 0; i < hostName.length(); i++) {
+                        // If we've got a dot we're at a new segment
+                        if (hostName.charAt(i) == '.') {
+                            count--;
+                            end = i;
+                        }
+                        // We've reached the precision we want
+                        if (count == 0) {
+                            break;
+                        }
+                    }
+                    if (end != 0 && count == 0) {
+                        return hostName.substring(0, end);
+                    }
+                }
+                return hostName;
             }
         };
     }
@@ -1073,9 +1101,13 @@ public final class Formatters {
     public static FormatStep mdcFormatStep(final String key, final boolean leftJustify, final int minimumWidth, final boolean truncateBeginning, final int maximumWidth) {
         return new JustifyingFormatStep(leftJustify, minimumWidth, truncateBeginning, maximumWidth) {
             public void renderRaw(final StringBuilder builder, final ExtLogRecord record) {
-                final String value = record.getMdc(key);
-                if (value != null) {
-                    builder.append(value);
+                if (key == null) {
+                    builder.append(new TreeMap<>(record.getMdcCopy()));
+                } else {
+                    final String value = record.getMdc(key);
+                    if (value != null) {
+                        builder.append(value);
+                    }
                 }
             }
         };
